Package: fromo
Type: Package
Maintainer: Steven E. Pav <shabbychef@gmail.com>
Authors@R: c(person(c("Steven", "E."), "Pav", 
    role=c("aut","cre"),
    email="shabbychef@gmail.com",
    comment = c(ORCID = "0000-0002-4197-6195")))
Version: 0.2.4
Date: 2024-11-29
License: LGPL-3
Title: Fast Robust Moments
BugReports: https://github.com/shabbychef/fromo/issues
Description: Fast, numerically robust computation of weighted moments via 'Rcpp'. 
   Supports computation on vectors and matrices, and Monoidal append of moments. 
   Moments and cumulants over running fixed length windows can be computed, 
   as well as over time-based windows.
   Moment computations are via a generalization of Welford's method, as described
   by Bennett et. (2009) <doi:10.1109/CLUSTR.2009.5289161>.
Imports: Rcpp (>= 0.12.3), methods
LinkingTo: Rcpp
Suggests: knitr, testthat, moments, PDQutils, e1071, microbenchmark
RoxygenNote: 7.3.2
URL: https://github.com/shabbychef/fromo
VignetteBuilder: knitr
Collate: 'fromo.r' 'RcppExports.R' 'zzz_centsums.R'
Author: Steven E. Pav [aut, cre] (<https://orcid.org/0000-0002-4197-6195>)
Built: R 4.4.2; x86_64-apple-darwin20.6.0; 2024-12-01 04:18:32 UTC; unix
